@@ -9,12 +9,12 @@
 
 //! A trait to represent zero-cost conversions.
 
-// TODO(ndmitchell): We could derive instances, similarly to `ref-cast`.
-// Leave that as future work if it turns out to be a useful idea.
-
 use crate::cast::{self, transmute_unchecked};
 pub use gazebo_derive::Coerce;
-use std::collections::{HashMap, HashSet};
+use std::{
+    alloc::Layout,
+    collections::{HashMap, HashSet},
+};
 
 /// A marker trait such that the existence of `From: Coerce<To>` implies
 /// that `From` can be treat as `To` without any data manipulation.
@@ -127,6 +127,7 @@ pub fn coerce<From, To>(x: From) -> To
 where
     From: Coerce<To>,
 {
+    assert_eq!(Layout::new::<From>(), Layout::new::<To>());
     unsafe { transmute_unchecked(x) }
 }
 
@@ -137,12 +138,14 @@ pub fn coerce_ref<From, To>(x: &From) -> &To
 where
     From: Coerce<To>,
 {
+    assert_eq!(Layout::new::<From>(), Layout::new::<To>());
     unsafe { cast::ptr(x) }
 }
 
 #[cfg(test)]
 mod test {
     use super::*;
+    use crate as gazebo;
 
     #[test]
     fn test_ptr_coerce() {
@@ -153,4 +156,14 @@ mod test {
         let x = "test".to_owned();
         assert_eq!(f(("test",)), (x.as_str(),))
     }
+
+    #[test]
+    fn test_coerce_lifetime() {
+        #[derive(Coerce)]
+        #[repr(transparent)]
+        struct NewtypeWithLifetime<'v>(&'v [usize]);
+
+        let newtype = NewtypeWithLifetime(&[1, 2]);
+        assert_eq!(&[1, 2], coerce(newtype))
+    }
 }
