@@ -10,6 +10,29 @@
 use crate::{dupe::Dupe, types::TEq};
 use std::borrow::Borrow;
 
+/// Optimised collect iterator into Vec, which might be a Result.
+///
+/// If we do a standard .collect() on the iterator it will never have a good size hint,
+/// as the lower bound will always be zero, so might reallocate several times.
+/// We know the Vec will either be thrown away, or exactly `len`, so aim if we do allocate,
+/// make sure it is at `len`. However, if the first element throws an error, we don't need
+/// to allocate at all, so special case that.
+fn collect_result<T, E>(mut it: impl ExactSizeIterator<Item = Result<T, E>>) -> Result<Vec<T>, E> {
+    match it.next() {
+        None => Ok(Vec::new()),
+        Some(Err(e)) => Err(e),
+        Some(Ok(x)) => {
+            // +1 for the element we have already consumed
+            let mut res = Vec::with_capacity(it.len() + 1);
+            res.push(x);
+            for x in it {
+                res.push(x?);
+            }
+            Ok(res)
+        }
+    }
+}
+
 /// Extension traits on slices/[`Vec`](Vec).
 pub trait SliceExt {
     type Item;
@@ -22,11 +45,9 @@ pub trait SliceExt {
     /// assert_eq!(vec![1,2,3].map(|x| x*x), vec![1,4,9]);
     /// ```
     ///
-    /// Note that from Rust 1.47.0 there is a `map` method on
-    /// arrays (e.g. `[T; N]`) behind the `array_map` feature flag.
-    /// Either enable this function (which would be `into_map` in our
-    /// vocabulary), or explicitly convert arrays to slices with the
-    /// `[..]` operation.
+    /// Note that from Rust 1.55.0 there is a `map` method on
+    /// arrays (e.g. `[T; N]`) so you'll need to explicitly convert
+    /// arrays to slices with the `[..]` operation.
     fn map<'a, B, F>(&'a self, f: F) -> Vec<B>
     where
         F: FnMut(&'a Self::Item) -> B;
@@ -110,7 +131,7 @@ impl<T> SliceExt for [T] {
     where
         F: FnMut(&'a Self::Item) -> Result<B, E>,
     {
-        self.iter().map(f).collect()
+        collect_result(self.iter().map(f))
     }
 
     fn as_singleton(&self) -> Option<&T> {
@@ -128,6 +149,7 @@ impl<T> SliceExt for [T] {
     }
 }
 
+// TODO(bobyf) merge these traits if we can properly implement linters without requiring separate traits
 /// Short hand analogous to `Iter::cloned`, where items of `&T` are converted to `T` via clone.
 ///
 /// ```
@@ -198,6 +220,41 @@ where
     }
 }
 
+/// Short hand analogous to `Iter::copied`, where items of `&T` are converted to `T` via `copy`.
+///
+/// ```
+/// use gazebo::prelude::*;
+///
+/// #[derive(Copy, Clone, Debug, PartialEq)]
+/// struct X;
+///
+/// let x = [&X];
+/// let y : Vec<X> = x.copied();
+///
+/// assert_eq!(y, vec![X]);
+///
+/// let x = vec![&X];
+/// let y : Vec<X> = x.copied();
+///
+/// assert_eq!(y, vec![X]);
+/// ```
+pub trait SliceCopiedExt {
+    type Item;
+
+    fn copied(&self) -> Vec<Self::Item>;
+}
+
+impl<T> SliceCopiedExt for [&T]
+where
+    T: Copy,
+{
+    type Item = T;
+
+    fn copied(&self) -> Vec<Self::Item> {
+        self.map(|x| **x)
+    }
+}
+
 /// Extension traits on [`Vec`](Vec).
 pub trait VecExt {
     type Item;
@@ -241,6 +298,6 @@ impl<T> VecExt for Vec<T> {
     where
         F: FnMut(Self::Item) -> Result<B, E>,
     {
-        self.into_iter().map(f).collect()
+        collect_result(self.into_iter().map(f))
     }
 }
